@@ -1,5 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables, FlexibleInstances #-}
-module Darcs.Test.HashedStorage( tests ) where
+module Darcs.Test.HashedStorage ( tests, unsafeMakeName ) where
 
 import Prelude hiding ( filter, readFile, writeFile, lookup, (<$>) )
 import qualified Prelude
@@ -13,7 +12,6 @@ import Control.Applicative( (<$>) )
 import Codec.Archive.Zip( extractFilesFromArchive, toArchive )
 
 import Data.Maybe
-import Data.Word
 import Data.List( sort, intercalate, intersperse )
 
 import Darcs.Util.Path hiding ( setCurrentDirectory )
@@ -61,17 +59,20 @@ dirs = [ floatPath "foo_dir"
 emptyStub :: TreeItem IO
 emptyStub = Stub (return emptyTree) NoHash
 
+unsafeMakeName :: String -> Name
+unsafeMakeName = either error id . makeName
+
 testTree :: Tree IO
 testTree =
-    makeTree [ (makeName "foo", emptyStub)
-             , (makeName "subtree", SubTree sub)
-             , (makeName "substub", Stub getsub NoHash) ]
-    where sub = makeTree [ (makeName "stub", emptyStub)
-                         , (makeName "substub", Stub getsub2 NoHash)
-                         , (makeName "x", SubTree emptyTree) ]
+    makeTree [ (unsafeMakeName "foo", emptyStub)
+             , (unsafeMakeName "subtree", SubTree sub)
+             , (unsafeMakeName "substub", Stub getsub NoHash) ]
+    where sub = makeTree [ (unsafeMakeName "stub", emptyStub)
+                         , (unsafeMakeName "substub", Stub getsub2 NoHash)
+                         , (unsafeMakeName "x", SubTree emptyTree) ]
           getsub = return sub
-          getsub2 = return $ makeTree [ (makeName "file", File emptyBlob)
-                                      , (makeName "file2",
+          getsub2 = return $ makeTree [ (unsafeMakeName "file", File emptyBlob)
+                                      , (unsafeMakeName "file2",
                                          File $ Blob (return $ BLC.pack "foo") NoHash) ]
 
 equals_testdata :: Tree IO -> IO ()
@@ -140,8 +141,6 @@ index :: [TF.Test]
 index = [ testCase "index versioning" check_index_versions
         , testCase "index listing" check_index
         , testCase "index content" check_index_content
-        , testProperty "xlate32" prop_xlate32
-        , testProperty "xlate64" prop_xlate64
         , testProperty "align bounded" prop_align_bounded
         , testProperty "align aligned" prop_align_aligned ]
     where pristine = readDarcsPristine "." >>= expand
@@ -169,8 +168,6 @@ index = [ testCase "index versioning" check_index_versions
                Prelude.writeFile "_darcs/index" "nonsense index... do not crash!"
                valid <- indexFormatValid "_darcs/index"
                assertBool "index format invalid" $ not valid
-          prop_xlate32 x = (xlate32 . xlate32) x == x where _types = x :: Word32
-          prop_xlate64 x = (xlate64 . xlate64) x == x where _types = x :: Word64
           prop_align_bounded (bound, x) =
               bound > 0 && bound < 1024 && x >= 0 ==>
                     align bound x >= x && align bound x < x + bound
@@ -200,7 +197,7 @@ tree = [ testCase "modifyTree" check_modify
        , testProperty "overlay keeps shape" prop_overlay_shape
        , testProperty "overlay is superset of over" prop_overlay_super ]
     where blob x = File $ Blob (return (BLC.pack x)) (sha256 $ BLC.pack x)
-          name = makeName
+          name = unsafeMakeName
           check_modify =
               let t = makeTree [(name "foo", blob "bar")]
                   modify = modifyTree t (floatPath "foo") (Just $ blob "bla")
@@ -378,7 +375,7 @@ instance (Monad m) => Arbitrary (TreeItem m) where
                       return (File $ Blob (return content) NoHash)
           subtree n = do branches <- choose (1, n)
                          let sub name = do t <- tree' ((n - 1) `div` branches)
-                                           return (makeName $ show name, t)
+                                           return (unsafeMakeName $ show name, t)
                          sublist <- mapM sub [0..branches]
                          oneof [ tree' 0
                                , return (SubTree $ makeTree sublist)
@@ -427,7 +424,7 @@ instance Show (Int -> Int) where
 shapeEq :: Tree m -> Tree m -> Bool
 shapeEq a b = Just EQ == cmpShape a b
 
-expandedShapeEq :: (Monad m, Functor m) => Tree m -> Tree m -> m Bool
+expandedShapeEq :: Monad m => Tree m -> Tree m -> m Bool
 expandedShapeEq a b = (Just EQ ==) <$> cmpExpandedShape a b
 
 cmpcat :: [Maybe Ordering] -> Maybe Ordering
@@ -438,7 +435,7 @@ cmpcat (x:y:rest) | x == y = cmpcat (x:rest)
 cmpcat [x] = x
 cmpcat [] = Just EQ -- empty things are equal
 
-cmpTree :: (Monad m, Functor m) => Tree m -> Tree m -> m (Maybe Ordering)
+cmpTree :: Monad m => Tree m -> Tree m -> m (Maybe Ordering)
 cmpTree x y = do x' <- expand x
                  y' <- expand y
                  con <- contentsEq x' y'
@@ -465,7 +462,7 @@ cmpExpandedShape a b = do x <- expand a
                           return $ x `cmpShape` y
 
 nondarcs :: AnchoredPath -> TreeItem m -> Bool
-nondarcs (AnchoredPath (x:_)) _ | x == makeName "_darcs" = False
+nondarcs (AnchoredPath (x:_)) _ | x == unsafeMakeName "_darcs" = False
                                      | otherwise = True
 nondarcs (AnchoredPath []) _ = True
 
