@@ -3,10 +3,8 @@ module Darcs.Patch.Prim.V1.Coalesce
     ()
     where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Prelude hiding ( pi )
 import Control.Arrow ( second )
 import Data.Maybe ( fromMaybe )
 import Data.Map ( elems, fromListWith, mapWithKey )
@@ -28,35 +26,17 @@ import Darcs.Patch.Witnesses.Ordered
     , reverseRL, mapFL, mapFL_FL
     , concatFL, lengthFL, (+>+) )
 import Darcs.Patch.Witnesses.Sealed
-    ( unseal, Sealed2(..), unsafeUnseal2
+    ( unseal, Sealed2(..), unseal2
     , Gap(..), unFreeLeft
     )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePEnd )
-import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Invert ( Invert(..), dropInverses )
 import Darcs.Patch.Commute ( Commute(..) )
 
 import Darcs.Util.Diff ( getChanges )
 import qualified Darcs.Util.Diff as D ( DiffAlgorithm )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Util.Path ( FileName, fp2fn )
-
--- | 'coalesceFwd' @p1 :> p2@ tries to combine @p1@ and @p2@ into a single
---   patch without intermediary changes.  For example, two hunk patches
---   modifying adjacent lines can be coalesced into a bigger hunk patch.
---   Or a patch which moves file A to file B can be coalesced with a
---   patch that moves file B into file C, yielding a patch that moves
---   file A to file C.
-coalesceFwd :: (Prim :> Prim) wX wY -> Maybe (FL Prim wX wY)
-coalesceFwd (FP f1 _ :> FP f2 _) | f1 /= f2 = Nothing
-coalesceFwd (p1 :> p2) | IsEq <- invert p1 =\/= p2 = Just NilFL
-coalesceFwd (FP f1 p1 :> FP _ p2) = fmap (:>: NilFL) $ coalesceFilePrim f1 (p1 :> p2) -- f1 = f2
-coalesceFwd (Move a b :> Move b' a') | b == b' = Just $ Move a a' :>: NilFL
-coalesceFwd (FP f AddFile :> Move a b) | f == a = Just $ FP b AddFile :>: NilFL
-coalesceFwd (DP f AddDir :> Move a b) | f == a = Just $ DP b AddDir :>: NilFL
-coalesceFwd (Move a b :> FP f RmFile) | b == f = Just $ FP a RmFile :>: NilFL
-coalesceFwd (Move a b :> DP f RmDir) | b == f = Just $ DP a RmDir :>: NilFL
-coalesceFwd (ChangePref p1 f1 t1 :> ChangePref p2 f2 t2) | p1 == p2 && t1 == f2 = Just $ ChangePref p1 f1 t2 :>: NilFL
-coalesceFwd _ = Nothing
+import Darcs.Util.Path ( AnchoredPath, floatPath )
 
 mapPrimFL :: (forall wX wY . FL Prim wX wY -> FL Prim wX wY)
              -> FL Prim wW wZ -> FL Prim wW wZ
@@ -71,36 +51,34 @@ mapPrimFL f x =
      Nothing -> f x
   where
         unsealList :: [Sealed2 p] -> FL p wA wB
-        unsealList = foldr ((:>:) . unsafeUnseal2) (unsafeCoerceP NilFL)
+        unsealList = foldr ((:>:) . unseal2 unsafeCoerceP) (unsafeCoerceP NilFL)
 
-        toSimpleSealed :: Sealed2 Prim -> Maybe (FileName, Sealed2 Simple)
+        toSimpleSealed :: Sealed2 Prim -> Maybe (AnchoredPath, Sealed2 Simple)
         toSimpleSealed (Sealed2 p) = fmap (second Sealed2) (toSimple p)
 
-
-
 data Simple wX wY
     = SFP !(FilePatchType wX wY)
     | SDP !(DirPatchType wX wY)
     | SCP String String String
     deriving ( Show )
 
-toSimple :: Prim wX wY -> Maybe (FileName, Simple wX wY)
+toSimple :: Prim wX wY -> Maybe (AnchoredPath, Simple wX wY)
 toSimple (FP a b) = Just (a, SFP b)
 toSimple (DP a AddDir) = Just (a, SDP AddDir)
 toSimple (DP _ RmDir) = Nothing -- ordering is trickier with rmdir present
 toSimple (Move _ _) = Nothing
-toSimple (ChangePref a b c) = Just (fp2fn $ darcsdir </> "prefs" </> "prefs", SCP a b c)
+toSimple (ChangePref a b c) = Just (floatPath (darcsdir </> "prefs" </> "prefs"), SCP a b c)
 
-fromSimple :: FileName -> Simple wX wY -> Prim wX wY
+fromSimple :: AnchoredPath -> Simple wX wY -> Prim wX wY
 fromSimple a (SFP b) = FP a b
 fromSimple a (SDP b) = DP a b
 fromSimple _ (SCP a b c) = ChangePref a b c
 
-fromSimples :: FileName -> FL Simple wX wY -> FL Prim wX wY
+fromSimples :: AnchoredPath -> FL Simple wX wY -> FL Prim wX wY
 fromSimples a = mapFL_FL (fromSimple a)
 
 tryHarderToShrink :: FL Prim wX wY -> FL Prim wX wY
-tryHarderToShrink x = tryToShrink2 $ fromMaybe x (tryShrinkingInverse x)
+tryHarderToShrink x = tryToShrink2 $ fromMaybe x (dropInverses x)
 
 tryToShrink2 :: FL Prim wX wY -> FL Prim wX wY
 tryToShrink2 psold =
@@ -125,7 +103,7 @@ tryOne :: RL Prim wW wX -> Prim wX wY -> FL Prim wY wZ
         -> Maybe (FL Prim wW wZ)
 tryOne _ _ NilFL = Nothing
 tryOne sofar p (p1:>:ps) =
-    case coalesceFwd (p :> p1) of
+    case coalesceOrCancel p p1 of
     Just p' -> Just (reverseRL sofar +>+ p' +>+ ps)
     Nothing -> case commute (p :> p1) of
                Nothing -> Nothing
@@ -159,10 +137,10 @@ pushCoalescePatch :: Prim wX wY -> FL Prim wY wZ
                     -> Either (FL Prim wX wZ) (FL Prim wX wZ)
 pushCoalescePatch new NilFL = Left (new:>:NilFL)
 pushCoalescePatch new ps@(p:>:ps')
-    = case coalesceFwd (new :> p) of
+    = case coalesceOrCancel new p of
       Just (new' :>: NilFL) -> Right $ either id id $ pushCoalescePatch new' ps'
       Just NilFL -> Right ps'
-      Just _ -> impossible -- coalesce either returns a singleton or empty
+      Just _ -> error "impossible case" -- coalesce either returns a singleton or empty
       Nothing -> if comparePrim new p == LT then Left (new:>:ps)
                             else case commute (new :> p) of
                                  Just (p' :> new') ->
@@ -172,21 +150,71 @@ pushCoalescePatch new ps@(p:>:ps')
                                      Left r -> Left (p' :>: r)
                                  Nothing -> Left (new:>:ps)
 
-coalesceFilePrim :: FileName -> (FilePatchType :> FilePatchType) wX wY
-                  -> Maybe (Prim wX wY)
-coalesceFilePrim f (Hunk line1 old1 new1 :> Hunk line2 old2 new2)
+coalesceOrCancel :: Prim wX wY -> Prim wY wZ -> Maybe (FL Prim wX wZ)
+coalesceOrCancel p1 p2
+  | IsEq <- invert p1 =\/= p2 = Just NilFL
+  | otherwise = fmap (:>: NilFL) $ coalescePair p1 p2
+
+-- | @'coalescePair' p1 p2@ tries to combine @p1@ and @p2@ into a single
+--   patch. For example, two hunk patches
+--   modifying adjacent lines can be coalesced into a bigger hunk patch.
+--   Or a patch which moves file A to file B can be coalesced with a
+--   patch that moves file B into file C, yielding a patch that moves
+--   file A to file C.
+coalescePair :: Prim wX wY -> Prim wY wZ -> Maybe (Prim wX wZ)
+coalescePair (FP f1 p1) (FP f2 p2)
+  | f1 /= f2 = Nothing
+  | otherwise = coalesceFilePrim f1 p1 p2
+coalescePair (Move a b) (Move b' c) | b == b' = Just $ Move a c
+coalescePair (FP a AddFile) (Move a' b) | a == a' = Just $ FP b AddFile
+coalescePair (DP a AddDir) (Move a' b) | a == a' = Just $ DP b AddDir
+coalescePair (Move a b) (FP b' RmFile) | b == b' = Just $ FP a RmFile
+coalescePair (Move a b) (DP b' RmDir) | b == b' = Just $ DP a RmDir
+{- we don't want to do that, of course:
+coalescePair (FP a RmFile) (FP b AddFile) | a == a' = Just $ Move a' b
+coalescePair (DP a RmDir) (DP b AddDir) | a == a' = Just $ Move a' b
+-}
+coalescePair (ChangePref p a b) (ChangePref p' b' c)
+  | p == p' && b == b' = Just $ ChangePref p a c
+coalescePair _ _ = Nothing
+
+-- | If 'coalescePair' is "addition" then this is "subtraction".
+decoalescePair :: Prim wX wZ -> Prim wX wY -> Maybe (Prim wY wZ)
+-- These two cases make sense only if we decoalesce;
+-- they correspond to the commented two cases for coalesce above
+-- and are one reason we need to define this function as a primitive
+decoalescePair (Move a b) (FP b' AddFile) | b == b' = Just (FP a RmFile)
+decoalescePair (Move a b) (DP b' AddDir) | b == b' = Just (DP a RmDir)
+decoalescePair (FP f1 p1) (FP f2 p2)
+  | f1 /= f2 = Nothing
+  | otherwise = decoalesceFilePrim f1 p1 p2
+decoalescePair z x = coalescePair (invert x) z
+
+coalesceFilePrim :: AnchoredPath -> FilePatchType wX wY -> FilePatchType wY wZ
+                 -> Maybe (Prim wX wZ)
+coalesceFilePrim f (Hunk line1 old1 new1) (Hunk line2 old2 new2)
     = coalesceHunk f line2 old2 new2 line1 old1 new1
 -- Token replace patches operating right after (or before) AddFile (RmFile)
 -- is an identity patch, as far as coalescing is concerned.
-coalesceFilePrim f (AddFile :> TokReplace{}) = Just $ FP f AddFile
-coalesceFilePrim f (TokReplace{} :> RmFile) = Just $ FP f RmFile
-coalesceFilePrim f (TokReplace t1 o1 n1 :> TokReplace t2 o2 n2)
-    | t1 == t2 && n1 == o2 = Just $ FP f $ TokReplace t1 o1 n2
-coalesceFilePrim f (Binary o m' :> Binary m n)
+-- These two cases make no sense when we decoalesce, which is the second
+-- reason decoalesce is defined as a primitive.
+coalesceFilePrim f (AddFile) (TokReplace{}) = Just $ FP f AddFile
+coalesceFilePrim f (TokReplace{}) (RmFile) = Just $ FP f RmFile
+coalesceFilePrim f (TokReplace t1 a b) (TokReplace t2 b' c)
+    | t1 == t2 && b == b' = Just $ FP f $ TokReplace t1 a c
+coalesceFilePrim f (Binary o m') (Binary m n)
     | m == m' = Just $ FP f $ Binary o n
-coalesceFilePrim _ _ = Nothing
+coalesceFilePrim _ _ _ = Nothing
 
-coalesceHunk :: FileName
+decoalesceFilePrim :: AnchoredPath -> FilePatchType wX wZ -> FilePatchType wX wY
+                   -> Maybe (Prim wY wZ)
+-- These two cases must fail because the token replace patches that coalesce
+-- has eliminated are irretrievably lost.
+decoalesceFilePrim _ (AddFile) (RmFile) = Nothing
+decoalesceFilePrim _ (RmFile) (TokReplace{}) = Nothing
+decoalesceFilePrim f z x = coalesceFilePrim f (invert x) z
+
+coalesceHunk :: AnchoredPath
              -> Int -> [B.ByteString] -> [B.ByteString]
              -> Int -> [B.ByteString] -> [B.ByteString]
              -> Maybe (Prim wX wY)
@@ -214,7 +242,7 @@ coalesceHunk f line1 old1 new1 line2 old2 new2
           lengthnew2 = length new2
 
 canonizeHunk :: Gap w
-             => D.DiffAlgorithm -> FileName -> Int -> [B.ByteString] -> [B.ByteString]
+             => D.DiffAlgorithm -> AnchoredPath -> Int -> [B.ByteString] -> [B.ByteString]
              -> w (FL Prim)
 canonizeHunk _ f line old new
     | null old || null new || old == [B.empty] || new == [B.empty]
@@ -222,7 +250,7 @@ canonizeHunk _ f line old new
 canonizeHunk da f line old new = makeHoley f line $ getChanges da old new
 
 makeHoley :: Gap w
-          => FileName -> Int -> [(Int,[B.ByteString], [B.ByteString])]
+          => AnchoredPath -> Int -> [(Int,[B.ByteString], [B.ByteString])]
           -> w (FL Prim)
 makeHoley f line =
     foldr (joinGap (:>:) . (\(l,o,n) -> freeGap (FP f (Hunk (l+line) o n)))) (emptyGap NilFL)
@@ -230,19 +258,15 @@ makeHoley f line =
 instance PrimCanonize Prim where
    tryToShrink = mapPrimFL tryHarderToShrink
 
-   tryShrinkingInverse (x:>:y:>:z)
-       | IsEq <- invert x =\/= y = Just z
-       | otherwise = case tryShrinkingInverse (y:>:z) of
-                     Nothing -> Nothing
-                     Just yz' -> Just $ fromMaybe (x :>: yz') $ tryShrinkingInverse (x:>:yz')
-   tryShrinkingInverse _ = Nothing
-
    sortCoalesceFL = mapPrimFL sortCoalesceFL2
    canonize _ p | IsEq <- isIdentity p = NilFL
    canonize da (FP f (Hunk line old new)) = unseal unsafeCoercePEnd $ unFreeLeft $ canonizeHunk da f line old new
    canonize _ p = p :>: NilFL
-   -- Running canonize twice is apparently necessary to fix issue525;
-   -- would be nice to understand why.
-   canonizeFL da = concatFL . mapFL_FL (canonize da) . sortCoalesceFL .
-                   concatFL . mapFL_FL (canonize da)
-   coalesce = coalesceFwd
+   -- Note: it is important to first coalesce and then canonize, since
+   -- coalescing can produce non-cononical hunks (while hunks resulting
+   -- from canonizing a single hunk cannot be coalesced). See issue525,
+   -- in particular msg20270 for details.
+   canonizeFL da = concatFL . mapFL_FL (canonize da) . sortCoalesceFL
+   coalesce (p1 :> p2) = coalesceOrCancel p1 p2
+   primCoalesce = coalescePair
+   primDecoalesce = decoalescePair
