@@ -24,7 +24,6 @@ module Darcs.UI.Commands.Util.Tree
     , treeHasAnycase
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad ( forM )
@@ -37,14 +36,14 @@ import qualified Darcs.Util.Tree.Monad as TM
 import Darcs.Util.Tree ( Tree, listImmediate, findTree )
 
 import Darcs.Util.Path
-    ( AnchoredPath(..), floatPath, eqAnycase )
+    ( AnchoredPath(..), eqAnycase )
 
 treeHasAnycase :: Monad m
                => Tree m
-               -> FilePath
+               -> AnchoredPath
                -> m Bool
 treeHasAnycase tree path =
-    fst `fmap` TM.virtualTreeMonad (existsAnycase $ floatPath path) tree
+    fst `fmap` TM.virtualTreeMonad (existsAnycase path) tree
 
 
 existsAnycase :: Monad m
@@ -53,7 +52,7 @@ existsAnycase :: Monad m
 existsAnycase (AnchoredPath []) = return True
 existsAnycase (AnchoredPath (x:xs)) = do
      wd <- TM.currentDirectory
-     tree <- fromMaybe (bug "invalid path passed to existsAnycase") <$>
+     tree <- fromMaybe (error "invalid path passed to existsAnycase") <$>
              gets (flip findTree wd . TM.tree)
      let subs = [ AnchoredPath [n] | (n, _) <- listImmediate tree,
                                           eqAnycase n x ]
@@ -63,11 +62,11 @@ existsAnycase (AnchoredPath (x:xs)) = do
                else TM.withDirectory path (existsAnycase $ AnchoredPath xs))
 
 
-treeHas :: Monad m => Tree m -> FilePath -> m Bool
-treeHas tree path = fst `fmap` TM.virtualTreeMonad (TM.exists $ floatPath path) tree
+treeHas :: Monad m => Tree m -> AnchoredPath -> m Bool
+treeHas tree path = fst `fmap` TM.virtualTreeMonad (TM.exists path) tree
 
-treeHasDir :: Monad m => Tree m -> FilePath -> m Bool
-treeHasDir tree path = fst `fmap` TM.virtualTreeMonad (TM.directoryExists $ floatPath path) tree
+treeHasDir :: Monad m => Tree m -> AnchoredPath -> m Bool
+treeHasDir tree path = fst `fmap` TM.virtualTreeMonad (TM.directoryExists path) tree
 
-treeHasFile :: Monad m => Tree m -> FilePath -> m Bool
-treeHasFile tree path = fst `fmap` TM.virtualTreeMonad (TM.fileExists $ floatPath path) tree
+treeHasFile :: Monad m => Tree m -> AnchoredPath -> m Bool
+treeHasFile tree path = fst `fmap` TM.virtualTreeMonad (TM.fileExists path) tree
