@@ -6,21 +6,21 @@ module Darcs.UI.PatchHeader
     , runHijackT
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Patch
-    ( IsRepoType, RepoPatch, PrimPatch, PrimOf, fromPrims
-    , effect
+    ( IsRepoType, RepoPatch, PrimPatch, PrimOf
     , summaryFL
     )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Info ( PatchInfo,
                           piAuthor, piName, piLog, piDateString,
-                          patchinfo, isInverted, invertName,
+                          patchinfo
                         )
-import Darcs.Patch.Named.Wrapped ( infopatch, getdeps, adddeps )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia, hopefully, info )
+import Darcs.Patch.Named
+   ( Named, patchcontents, patch2patchinfo, infopatch, getdeps, adddeps
+   )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia )
 import Darcs.Patch.Prim ( canonizeFL )
 
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+) )
@@ -51,7 +51,8 @@ import Control.Exception ( catch, IOException )
 import Control.Monad ( when, void )
 import Control.Monad.Trans              ( liftIO )
 import Control.Monad.Trans.State.Strict ( StateT(..), evalStateT, get, put  )
-import Data.List ( isPrefixOf )
+import Data.List ( isPrefixOf, stripPrefix )
+import Data.Maybe ( fromMaybe )
 import System.Exit ( exitSuccess )
 import System.IO ( stdin )
 
@@ -89,10 +90,12 @@ getLog :: forall prim wX wY . PrimPatch prim
        -> FL prim wX wY                         -- ^ changes to record
        -> IO (String, [String], Maybe String)   -- ^ patch name, long description and possibly the path
                                                 --   to the temporary file that should be removed later
-getLog m_name has_pipe log_file ask_long m_old chs = go has_pipe log_file ask_long where
+getLog m_name has_pipe log_file ask_long m_old chs =
+  restoreTagPrefix <$> go has_pipe log_file ask_long
+ where
   go True _ _ = do
       p <- case patchname_specified of
-             FlagPatchName p  -> return p
+             FlagPatchName p  -> check_badname p >> return p
              PriorPatchName p -> return p
              NoPatchName      -> prompt_patchname False
       putStrLn "What is the log?"
@@ -101,7 +104,7 @@ getLog m_name has_pipe log_file ask_long m_old chs = go has_pipe log_file ask_lo
   go _ (O.Logfile { O._logfile = Just f }) _ = do
       mlp <- readTextFile f `catch` (\(_ :: IOException) -> return [])
       firstname <- case (patchname_specified, mlp) of
-                     (FlagPatchName  p, []) -> return p
+                     (FlagPatchName  p, []) -> check_badname p >> return p
                      (_, p:_)               -> if badName p
                                                  then prompt_patchname True
                                                  else return p -- logfile trumps prior!
@@ -113,64 +116,82 @@ getLog m_name has_pipe log_file ask_long m_old chs = go has_pipe log_file ask_lo
       return (name, thelog, if O._rmlogfile log_file then Just $ toFilePath f else Nothing)
   go _ _ (Just O.YesEditLongComment) =
       case patchname_specified of
-          FlagPatchName  p  -> actually_get_log p
-          PriorPatchName p  -> actually_get_log p
-          NoPatchName       -> actually_get_log ""
+          FlagPatchName  p  -> get_log_using_editor p
+          PriorPatchName p  -> get_log_using_editor p
+          NoPatchName       -> get_log_using_editor ""
   go _ _ (Just O.NoEditLongComment) =
       case patchname_specified of
-          FlagPatchName  p  -> return (p, default_log, Nothing) -- record (or amend) -m
+          FlagPatchName  p  -> check_badname p >> return (p, default_log, Nothing) -- record (or amend) -m
           PriorPatchName p  -> return (p, default_log, Nothing) -- amend
           NoPatchName       -> do p <- prompt_patchname True -- record
                                   return (p, [], Nothing)
   go _ _ (Just O.PromptLongComment) =
       case patchname_specified of
-          FlagPatchName p   -> prompt_long_comment p -- record (or amend) -m
+          FlagPatchName p   -> check_badname p >> prompt_long_comment p -- record (or amend) -m
           PriorPatchName p  -> prompt_long_comment p
           NoPatchName       -> prompt_patchname True >>= prompt_long_comment
   go _ _ Nothing =
       case patchname_specified of
-          FlagPatchName  p  -> return (p, default_log, Nothing)  -- record (or amend) -m
-          PriorPatchName "" -> actually_get_log ""
+          FlagPatchName  p  -> check_badname p >> return (p, default_log, Nothing)  -- record (or amend) -m
+          PriorPatchName "" -> get_log_using_editor ""
           PriorPatchName p  -> return (p, default_log, Nothing)
-          NoPatchName       -> actually_get_log ""
+          NoPatchName       -> get_log_using_editor ""
 
-  patchname_specified = case (m_name, m_old) of
-                          (Just name, _) | badName name -> NoPatchName
-                                         | otherwise    -> FlagPatchName name
-                          (Nothing,   Just (name,_))    -> PriorPatchName name
-                          (Nothing,   Nothing)          -> NoPatchName
+  tagPrefix = "TAG "
 
-  badName "" = True
-  badName n  = "TAG" `isPrefixOf` n
+  hasTagPrefix name = tagPrefix `isPrefixOf` name
+
+  restoreTagPrefix (name, log, file)
+    | Just (old_name, _) <- m_old
+    , hasTagPrefix old_name = (tagPrefix ++ name, log, file)
+  restoreTagPrefix args = args
+
+  stripTagPrefix name = fromMaybe name $ stripPrefix tagPrefix name
+
+  patchname_specified =
+    case (m_name, m_old) of
+      (Just name, _)              -> FlagPatchName name
+      (Nothing,   Just (name, _)) -> PriorPatchName (stripTagPrefix name)
+      (Nothing,   Nothing)        -> NoPatchName
+
+  badName n = null n || hasTagPrefix n
 
   default_log = case m_old of
                   Nothing    -> []
                   Just (_,l) -> l
 
-  prompt_patchname retry =
-    do n <- askUser "What is the patch name? "
-       if badName n
-          then if retry then prompt_patchname retry
-                        else fail "Bad patch name!"
-          else return n
+  check_badname = maybe (return ()) fail . just_a_badname
+
+  prompt_patchname retry = do
+      n <- askUser "What is the patch name? "
+      maybe (return n) prompt_again $ just_a_badname n
+    where
+      prompt_again msg = do
+        putStrLn msg
+        if retry then prompt_patchname retry else fail "Bad patch name!"
+
+  just_a_badname n =
+    if null n then
+      Just "The patch name must not be empty!"
+    else if hasTagPrefix n then
+      Just "The patch name must not start with \"TAG \"!"
+    else
+      Nothing
 
   prompt_long_comment oldname =
     do y <- promptYorn "Do you want to add a long comment?"
-       if y then actually_get_log oldname
+       if y then get_log_using_editor oldname
             else return (oldname, [], Nothing)
 
-  actually_get_log p = do let logf = darcsLastMessage
+  get_log_using_editor p =
+                       do let logf = darcsLastMessage
                           -- TODO: make sure encoding used for logf is the same everywhere
                           -- probably should be locale because the editor will assume it
                           writeTextFile logf $ unlines $ p : default_log
                           append_info logf p
                           _ <- editFile logf
                           (name,long) <- read_long_comment logf p
-                          if badName name
-                            then do putStrLn "WARNING: empty or incorrect patch name!"
-                                    pn <- prompt_patchname True
-                                    return (pn, long, Nothing)
-                            else return (name,long,Just logf)
+                          return (name,long,Just logf)
 
   read_long_comment :: FilePathLike p => p -> String -> IO (String, [String])
   read_long_comment f oldname =
@@ -178,7 +199,9 @@ getLog m_name has_pipe log_file ask_long m_old chs = go has_pipe log_file ask_lo
          let filter_out_info = filter (not.("#" `isPrefixOf`))
          case reverse $ dropWhile null $ reverse $ filter_out_info t of
             []     -> return (oldname, [])
-            (n:ls) -> return (n, ls)
+            (n:ls) -> do
+                check_badname n
+                return (n, ls)
 
   append_info f oldname = do
     fc <- readTextFile f
@@ -216,29 +239,31 @@ updatePatchHeader :: forall rt p wX wY wR wU wT . (IsRepoType rt, RepoPatch p, A
                   -> Maybe String -- author
                   -> Maybe String -- patchname
                   -> Maybe O.AskLongComment
-                  -> PatchInfoAnd rt p wT wX
-                  -> FL (PrimOf p) wX wY
+                  -> Named (PrimOf p) wT wX
+                  -- ^ patch to edit, must be conflict-free as conflicts can't be preserved when changing
+                  -- the identity of a patch. If necessary this can be achieved by calling @fmapFL_Named effect@
+                  -- on an @Named p@ first, but some callers might already have @Named (PrimOf p)@ available.
+                  -> FL (PrimOf p) wX wY -- ^new primitives to add
                   -> HijackT IO (Maybe String, PatchInfoAnd rt p wT wY)
 updatePatchHeader verb ask_deps pSelOpts da nKeepDate nSelectAuthor nAuthor nPatchname nAskLongComment oldp chs = do
 
-    let newchs = canonizeFL da (effect oldp +>+ chs)
+    let newchs = canonizeFL da (patchcontents oldp +>+ chs)
 
-    let old_pdeps = getdeps $ hopefully oldp
+    let old_pdeps = getdeps oldp
     newdeps <-
         case ask_deps of
            AskAboutDeps repository -> liftIO $ askAboutDepends repository newchs pSelOpts old_pdeps
            NoAskAboutDeps -> return old_pdeps
 
-    let old_pinf = info oldp
+    let old_pinf = patch2patchinfo oldp
         prior    = (piName old_pinf, piLog old_pinf)
     date <- if nKeepDate then return (piDateString old_pinf) else liftIO $ getDate False
     new_author <- getAuthor verb nSelectAuthor nAuthor old_pinf
     liftIO $ do
         (new_name, new_log, mlogf) <- getLog
             nPatchname False (O.Logfile Nothing False) nAskLongComment (Just prior) chs
-        let maybe_invert = if isInverted old_pinf then invertName else id
-        new_pinf <- maybe_invert `fmap` patchinfo date new_name new_author new_log
-        let newp = n2pia (adddeps (infopatch new_pinf (fromPrims newchs)) newdeps)
+        new_pinf <- patchinfo date new_name new_author new_log
+        let newp = n2pia (adddeps (infopatch new_pinf newchs) newdeps)
         return (mlogf, newp)
 
 
