@@ -30,14 +30,12 @@ module Darcs.Util.ByteString
     , gzDecompress
     -- * list utilities
     , dropSpace
-    , breakSpace
     , linesPS
     , unlinesPS
     , hashPS
     , breakFirstPS
     , breakLastPS
     , substrPS
-    , readIntPS
     , isFunky
     , fromHex2PS
     , fromPS2Hex
@@ -57,7 +55,6 @@ module Darcs.Util.ByteString
     , spec_betweenLinesPS
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Codec.Binary.Base16 ( b16Enc, b16Dec )
@@ -75,7 +72,7 @@ import System.IO.Error          ( catchIOError )
 import System.IO.Unsafe         ( unsafePerformIO )
 
 import Data.Bits                ( rotateL )
-import Data.Char                ( ord, isSpace, toLower, toUpper )
+import Data.Char                ( ord, toLower, toUpper )
 import Data.Word                ( Word8 )
 import Data.Int                 ( Int32, Int64 )
 import Data.List                ( intersperse )
@@ -94,20 +91,15 @@ import System.IO.MMap( mmapFileByteString )
 import System.Mem( performGC )
 import System.Posix.Files( fileSize, getSymbolicLinkStatus )
 
--- | readIntPS skips any whitespace at the beginning of its argument, and
--- reads an Int from the beginning of the PackedString.  If there is no
--- integer at the beginning of the string, it returns Nothing, otherwise it
--- just returns the int read, along with a B.ByteString containing the
--- remainder of its input.
-
-readIntPS :: B.ByteString -> Maybe (Int, B.ByteString)
-readIntPS = BC.readInt . BC.dropWhile isSpace
-
 ------------------------------------------------------------------------
 -- A locale-independent isspace(3) so patches are interpreted the same everywhere.
 -- ((c) == ' ' || (c) == '\t' || (c) == '\n' || (c) == '\r')
 isSpaceWord8 :: Word8 -> Bool
-isSpaceWord8 = (`elem` [0x20, 0x09, 0x0A, 0x0D])
+isSpaceWord8 0x20 = True
+isSpaceWord8 0x09 = True
+isSpaceWord8 0x0A = True
+isSpaceWord8 0x0D = True
+isSpaceWord8 _    = False
 {-# INLINE isSpaceWord8 #-}
 
 -- | Drop leading white space, where white space is defined as
@@ -115,10 +107,6 @@ isSpaceWord8 = (`elem` [0x20, 0x09, 0x0A, 0x0D])
 dropSpace :: B.ByteString -> B.ByteString
 dropSpace bs = B.dropWhile isSpaceWord8 bs
 
--- | Split at first occurrence of ' ', '\t', '\n', or '\r'.
-breakSpace :: B.ByteString -> (B.ByteString, B.ByteString)
-breakSpace bs = B.break isSpaceWord8 bs
-
 ------------------------------------------------------------------------
 
 {-# INLINE isFunky #-}
