@@ -21,7 +21,6 @@ module Darcs.Util.Lock
     , environmentHelpLocks
     , withTemp
     , withOpenTemp
-    , withStdoutTemp
     , withTempDir
     , withPermDir
     , withDelayedDir
@@ -39,7 +38,6 @@ module Darcs.Util.Lock
     , gzWriteAtomicFilePS
     , gzWriteAtomicFilePSs
     , gzWriteDocFile
-    , rmRecursive
     , removeFileMayNotExist
     , canonFilename
     , maybeRelink
@@ -50,7 +48,6 @@ module Darcs.Util.Lock
     , withNewDirectory
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Data.List ( inits )
@@ -75,17 +72,18 @@ import Control.Exception
     , SomeException
     )
 import System.Directory
-    ( removeFile
-    , removeDirectory
+    ( removePathForcibly
     , doesFileExist
     , doesDirectoryExist
-    , getDirectoryContents
     , createDirectory
     , getTemporaryDirectory
-    , removeDirectoryRecursive
+    , removePathForcibly
+    , renameFile
+    , renameDirectory
     )
-import System.FilePath.Posix ( splitDirectories )
+import System.FilePath.Posix ( splitDirectories, splitFileName )
 import System.Environment ( lookupEnv )
+import System.IO.Temp ( createTempDirectory )
 
 import Control.Concurrent ( threadDelay )
 import Control.Monad ( unless, when, liftM )
@@ -100,7 +98,7 @@ import Darcs.Util.Exception
     , catchall
     )
 import Darcs.Util.File ( withCurrentDirectory
-                       , doesDirectoryReallyExist, removeFileMayNotExist )
+                       , removeFileMayNotExist )
 import Darcs.Util.Path ( AbsolutePath, FilePathLike, toFilePath,
                         getCurrentDirectory, setCurrentDirectory )
 
@@ -111,10 +109,8 @@ import Darcs.Util.SignalHandler ( withSignalsBlocked )
 import Darcs.Util.Printer ( Doc, hPutDoc, packedString, empty, renderPSs )
 import Darcs.Util.AtExit ( atexit )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Util.Workaround ( renameFile )
 import Darcs.Util.Compat
-    ( mkStdoutTemp
-    , canonFilename
+    ( canonFilename
     , maybeRelink
     , atomicCreate
     , sloppyAtomicCreate
@@ -202,9 +198,6 @@ withOpenTemp = bracket get_empty_file cleanup
           get_empty_file = invert `fmap` openBinaryTempFile "." "darcs"
           invert (a,b) = (b,a)
 
-withStdoutTemp :: (FilePath -> IO a) -> IO a
-withStdoutTemp = bracket (mkStdoutTemp "stdout_") removeFileMayNotExist
-
 tempdirLoc :: IO FilePath
 tempdirLoc = liftM fromJust $
     firstJustIO [ liftM (Just . head . words) (readFile (darcsdir++"/prefs/tmpdir")) >>= chkdir,
@@ -240,31 +233,36 @@ data WithDirKind = Perm | Temp | Delayed
 withDir :: WithDirKind  -- specifies if and when directory will be deleted
         -> FilePath     -- path parameter
         -> (AbsolutePath -> IO a) -> IO a
-withDir _ "" _ = bug "withDir called with empty directory name"
+withDir _ "" _ = error "withDir called with empty directory name"
 withDir kind absoluteOrRelativeName job = do
   absoluteName <- if isRelative absoluteOrRelativeName
                    then fmap (++ absoluteOrRelativeName) tempdirLoc
                    else return absoluteOrRelativeName
   formerdir <- getCurrentDirectory
-  bracket (createDir absoluteName 0)
-          (\dir -> do setCurrentDirectory formerdir
-                      k <- keepTempDir
-                      unless k $ case kind of
-                                   Perm -> return ()
-                                   Temp -> rmRecursive (toFilePath dir)
-                                   Delayed -> atexit $ rmRecursive (toFilePath dir))
+  bracket (createDir absoluteName)
+          (\dir -> do
+              setCurrentDirectory formerdir
+              k <- keepTempDir
+              unless k $
+                  case kind of
+                      Perm -> return ()
+                      Temp -> cleanup (toFilePath dir)
+                      Delayed -> atexit $ cleanup (toFilePath dir))
           job
-    where newname name 0 = name
-          newname name n = name ++ "-" ++ show n
-          createDir :: FilePath -> Int -> IO AbsolutePath
-          createDir name n
-              = do createDirectory $ newname name n
-                   setCurrentDirectory $ newname name n
+    where createDir :: FilePath -> IO AbsolutePath
+          createDir name
+              = do let (parent,dir) = splitFileName name
+                   createTempDirectory parent dir >>= setCurrentDirectory
                    getCurrentDirectory
-                `catch` (\e -> if isAlreadyExistsError e
-                               then createDir name (n+1)
-                               else throwIO e)
           keepTempDir = isJust `fmap` lookupEnv "DARCS_KEEP_TMPDIR"
+          toDelete dir = dir ++ "_done"
+          cleanup path = do
+              -- so asynchronous threads cannot add any more
+              -- files while we are deleting
+              debugMessage $ unwords ["atexit: renaming",path,"to",toDelete path]
+              renameDirectory path (toDelete path)
+              debugMessage $ unwords ["atexit: deleting",toDelete path]
+              removePathForcibly (toDelete path)
 
 environmentHelpKeepTmpdir :: ([String], [String])
 environmentHelpKeepTmpdir = (["DARCS_KEEP_TMPDIR"],[
@@ -300,19 +298,6 @@ withTempDir = withDir Temp
 withDelayedDir :: FilePath -> (AbsolutePath -> IO a) -> IO a
 withDelayedDir = withDir Delayed
 
-rmRecursive :: FilePath -> IO ()
-rmRecursive d =
-    do isd <- doesDirectoryReallyExist d
-       if not isd
-          then removeFile d
-          else do conts <- actual_dir_contents
-                  withCurrentDirectory d $
-                    mapM_ rmRecursive conts
-                  removeDirectory d
-    where actual_dir_contents = -- doesn't include . or ..
-              do c <- getDirectoryContents d
-                 return $ filter (/=".") $ filter (/="..") c
-
 worldReadableTemp :: FilePath -> IO FilePath
 worldReadableTemp f = wrt 0
     where wrt :: Int -> IO FilePath
@@ -417,5 +402,5 @@ withNewDirectory :: FilePath -> IO () -> IO ()
 withNewDirectory name action = do
   createDirectory name
   withCurrentDirectory name action `catch` \e -> do
-    removeDirectoryRecursive name `catchIOError` (const $ return ())
+    removePathForcibly name `catchIOError` (const $ return ())
     throwIO (e :: SomeException)
