@@ -1,4 +1,4 @@
-#!/bin/env bash
+#!/usr/bin/env bash
 #
 # Test darcs conflict fight scenario.
 #
@@ -11,7 +11,10 @@
 
 . ./lib
 
-record="record --ignore-time --all --author X"
+# test fails for these obsolete formats:
+skip-formats darcs-1 darcs-2
+
+num_conflicts=40
 
 rm -rf RA RB
 mkdir RA
@@ -20,7 +23,7 @@ cd RA
 echo 0 > file
 darcs init
 darcs add file
-darcs $record -m0 file
+darcs record -am0 file
 cd ..
 
 darcs get RA RB
@@ -28,27 +31,31 @@ darcs get RA RB
 # Create conflict in RB
 cd RB
 echo let it b > file
-darcs $record -m B
+darcs record -am B
 cd ..
 
-for i in 1 2 3 4 5 # 6 7 8 9 10 11 12
+for i in $(seq 1 $num_conflicts)
 do
-  echo Create new patch A$i in RA
   cd RA
+  echo Create new patch A$i in RA
   echo a$i > file
-  darcs $record -m A$i
+  darcs record -am A$i
   cd ..
 
-  echo Pull patch A$i from RA and get a conflict
   cd RB
-  time darcs pull ../RA --verbose --all --patch "^A$i\$"
-  cd ..
+  echo Pull patch A$i from RA and get a conflict
+
+  /usr/bin/env time -f %e -o ../elapsed darcs pull ../RA --quiet --all --patch "^A$i\$" --allow-conflicts
+  if (( $i == 1 )); then
+    start=$(cat ../elapsed)
+  else
+    elapsed=$(cat ../elapsed)
+    # check that the runtime is not more than quadratic in i
+    (( $(echo "$elapsed < 1.5 * $i * $i * $start" | bc) ))
+  fi
 
   echo Resolve conflict and start fighting by recording B$i
-  cd RB
   echo let it b > file
-  darcs $record -m B$i
+  darcs record -am B$i
   cd ..
 done
-
-rm -rf RA RB
