@@ -32,11 +32,47 @@ cd R
 darcs init
 echo 'wibble' > wibble
 darcs rec -lam "add wibble"
+darcs tag bla
 cd ..
 
+# suspend, repull, rebase obliterate
+
+rm -rf R1
+darcs get R R1
+
+cd R1
+echo yy | darcs rebase suspend
+echo yy | darcs rebase suspend
+echo yd | darcs pull ../R
+echo yd | darcs rebase obliterate
+echo yd | darcs rebase obliterate
+cd ..
+
+# issue2445: suspend, repull, unsuspend
+
+rm -rf R2
 darcs get R R2
 
-cd R
-echo 'yy' | darcs rebase suspend
-darcs pull -a ../R2
-echo 'yy' | darcs rebase obliterate
+cd R2
+echo yy | darcs rebase suspend
+echo yy | darcs rebase suspend
+echo yd | darcs pull -a ../R
+echo yd | darcs rebase unsuspend --allow-conflicts
+echo yd | darcs pull -a ../R
+echo yd | darcs rebase unsuspend
+cd ..
+
+# suspend, repull, suspend again, unsuspend
+
+rm -rf R3
+darcs get R R3
+
+cd R3
+echo yy | darcs rebase suspend
+echo yy | darcs rebase suspend
+echo yd | darcs pull -a ../R
+echo yd | darcs pull -a ../R
+echo yy | darcs rebase suspend
+echo yy | darcs rebase suspend
+echo wwyd | darcs rebase unsuspend
+cd ..
