@@ -54,10 +54,13 @@ darcs record  -a -m 'second record' --logfile=log.txt  date.t | grep -i 'finishe
 
 # refuse empty patch name
 export DARCS_EDITOR="cat -n"
-date >> date.t
-echo "patchname" | darcs record -a -m ""  | grep WARNING
-date >> date.t
-darcs record -a -m "some name"
+echo "patchname" | not darcs record -a -m ""
+not darcs record -a -m ""
+
+# refuse patch name starting with "TAG "
+export DARCS_EDITOR="cat -n"
+echo "patchname" | not darcs record -a -m "TAG fake"
+not darcs record -a -m "TAG fake"
 
 cd ..
 rm -rf temp1
